/* ----------------------------------------------------------------- */
/*           The HMM-Based Singing Voice Synthesis System "Sinsy"    */
/*           developed by Sinsy Working Group                        */
/*           http://sinsy.sourceforge.net/                           */
/* ----------------------------------------------------------------- */
/*                                                                   */
/*  Copyright (c) 2009-2015  Nagoya Institute of Technology          */
/*                           Department of Computer Science          */
/*                                                                   */
/* All rights reserved.                                              */
/*                                                                   */
/* Redistribution and use in source and binary forms, with or        */
/* without modification, are permitted provided that the following   */
/* conditions are met:                                               */
/*                                                                   */
/* - Redistributions of source code must retain the above copyright  */
/*   notice, this list of conditions and the following disclaimer.   */
/* - Redistributions in binary form must reproduce the above         */
/*   copyright notice, this list of conditions and the following     */
/*   disclaimer in the documentation and/or other materials provided */
/*   with the distribution.                                          */
/* - Neither the name of the Sinsy working group nor the names of    */
/*   its contributors may be used to endorse or promote products     */
/*   derived from this software without specific prior written       */
/*   permission.                                                     */
/*                                                                   */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND            */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,       */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF          */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE          */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,          */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED   */
/* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     */
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON */
/* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   */
/* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    */
/* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE           */
/* POSSIBILITY OF SUCH DAMAGE.                                       */
/* ----------------------------------------------------------------- */

#ifndef SINSY_CONFIGURATIONS_H_
#define SINSY_CONFIGURATIONS_H_

#include <map>
#include <string>
#include <sstream>
#include <algorithm>
#include "util_types.h"

namespace sinsy
{

class Configurations
{
public:
   //! constructor
   Configurations();

   //! destructor
   virtual ~Configurations();

   //! clear
   void clear();

   //! read configurations from file
   bool read(const std::string& fpath);

   //! get
   template<class T> T get(const std::string&, const T&) const;

   //! get
   std::string get(const std::string&) const;

private:
   //! copy constructor (donot use)
   Configurations(const Configurations&);

   //! assignment operator (donot use)
   Configurations& operator=(const Configurations&);

   typedef std::map<std::string, std::string> Configs;

   //! configs
   Configs configs;
};

/*!
 get

 @param key
 @param def default value
 @return
 */
template<class T>
T Configurations::get(const std::string& key, const T& def) const
{
   T ret(def);
   Configs::const_iterator itr(configs.find(key));
   if (configs.end() != itr) {
      std::istringstream iss(itr->second);
      iss >> ret;
   }
   return ret;
}

/*!
 get (std::string)
 */
template<>
std::string Configurations::get<std::string>(const std::string& key, const std::string& def) const;

/*!
 get (bool)
 */
template<>
bool Configurations::get<bool>(const std::string& key, const bool& def) const;

};

#endif // SINSY_CONFIGURATIONS_H_
